﻿///////////////////////////////////////////////////////////////////////////////
//
// CsvWriterクラス
//  
// Copyright (c) 2016, Advanced Core Technologies, Inc. All Rights Reserved.
//
// 2016/01/15  Rev.1.0.0.0  MOTO    作成
//
///////////////////////////////////////////////////////////////////////////////

using System;
using System.Collections.Generic;
using System.IO;

namespace ACT.Utilities.CsvIO
{
    /// <summary>
    /// CSVをストリームや文字列に書き込みます。
    /// </summary>
    public class CsvWriter : IDisposable
    {
        //=========================================================================================
        //  Constructors
        //=========================================================================================
        #region Constructors

        /// <summary>
        /// 指定したTextWriterを使用して初期化します。
        /// </summary>
        /// <param name="writer">書き込みを行うTextWriter</param>
        public CsvWriter(TextWriter writer)
        {
            Writer = writer;
        }

        #endregion

        //=========================================================================================
        //  Private Fields
        //=========================================================================================
        #region Private Fields

        /// <summary>
        /// Dispose済みフラグです。
        /// </summary>
        private bool alreadyDisposed_;

        #endregion

        //=========================================================================================
        //  Properties
        //=========================================================================================
        #region Properties

        /// <summary>
        /// 書き込みを行うTextWriterを取得します。
        /// </summary>
        public TextWriter Writer { get; private set; }

        #endregion

        //=========================================================================================
        //  Public Methods
        //=========================================================================================
        #region Public Methods

        /// <summary>
        /// 指定された値をストリームまたは文字列へ書き込みます。
        /// </summary>
        /// <param name="value">書き込む文字列</param>
        /// <param name="appendNewLine">末尾に改行文字列を書き込む場合はtrue、既定値はfalseです。</param>
        public void WriteField(string value, bool appendNewLine = false)
        {
            if (value == null)
            {
                throw new ArgumentNullException("value");
            }

            string writeString = value;
            if (appendNewLine)
            {
                writeString += Environment.NewLine;
            }
            Writer.Write(writeString);
        }

        /// <summary>
        /// 指定されたレコードと末尾に改行コードをストリームまたは文字列へ書き込みます。
        /// </summary>
        /// <param name="record">書き込むCSVレコード</param>
        public void WriteRecord(CsvRecord record)
        {
            if (record == null)
            {
                throw new ArgumentNullException("record");
            }

            Writer.WriteLine(record);
        }

        /// <summary>
        /// 指定された複数のレコードをストリームまたは文字列へ書き込みます。
        /// </summary>
        /// <param name="records">書き込むCSVレコードのコレクション</param>
        public void WriteRecords(IEnumerable<CsvRecord> records)
        {
            if (records == null)
            {
                throw new ArgumentNullException("records");
            }

            foreach (var record in records)
            {
                WriteRecord(record);
            }
        }

        #endregion

        //=========================================================================================
        //  IDisposable
        //=========================================================================================
        #region IDisposable

        /// <summary>
        /// このCsvWriterオブジェクトによって使用されている全てのリソースを解放します。
        /// </summary>
        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }

        /// <summary>
        /// 全てのリソースを解放します。
        /// </summary>
        /// <param name="disposing"></param>
        protected virtual void Dispose(bool disposing)
        {
            if (alreadyDisposed_)
            {
                return;
            }

            if (disposing)
            {
                // マネージリソースの解放
                Writer.Dispose();

            }
            // アンマネージリソースの解放

            alreadyDisposed_ = true;
        }

        /// <summary>
        /// デストラクターです。
        /// </summary>
        ~CsvWriter()
        {
            Dispose(false);
        }

        #endregion
    }
}
