﻿///////////////////////////////////////////////////////////////////////////////////////////////////
// 
//  ACT距離計算サービス サンプルプログラム
//  MainFormクラス
//
//  Copyrights (c) 2016-2023, Advanced Core Technologies, Inc. All rights reserved.
//
//  2023/09/21 Rev.1.0.0.1 KYON AdminURLに合わせてSSL使用フラグを変更出来るように変更
//  2016/01/21 Rev.1.0.0.0 MOTO 作成
//
///////////////////////////////////////////////////////////////////////////////////////////////////

using ACT.Utilities.CsvIO;
using System;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Windows.Forms;

namespace ACTSOAPCSCalcRoute
{
    public partial class MainForm : Form
    {
        //=========================================================================================
        //  Constructors
        //=========================================================================================
        #region Constructors

        /// <summary>
        /// 既定値を使用して初期化します。
        /// </summary>
        public MainForm()
        {
            InitializeComponent();
        }

        #endregion

        //=========================================================================================
        //  Private Fields
        //=========================================================================================
        #region Private Fields

        /// <summary>
        /// ユーザー情報です。
        /// </summary>
        private UserSettings userSettings_;

        /// <summary>
        /// 距離計算サービスのユーザーです。
        /// </summary>
        private User user_;

        #endregion

        //=========================================================================================
        //  Private Constants
        //=========================================================================================
        #region Private Constants

        /// <summary>
        /// 地点名の列番号です。
        /// </summary>
        private static readonly int NameColumnIndex = 0;

        /// <summary>
        /// 住所の列番号です。
        /// </summary>
        private static readonly int AddressColumnIndex = 1;

        /// <summary>
        /// 経度の列番号です。
        /// </summary>
        private static readonly int LonColumnIndex = 2;

        /// <summary>
        /// 緯度の列番号です。
        /// </summary>
        private static readonly int LatColumnIndex = 3;

        #endregion

        //=========================================================================================
        //  Private Methods
        //=========================================================================================
        #region Private Methods

        /// <summary>
        /// 計算を実行し実行結果をファイルに出力します。
        /// </summary>
        /// <param name="inputFilePath">入力ファイルのパス</param>
        /// <param name="outputFilePath">出力ファイルのパス</param>
        /// <param name="method">ルート計算の関数</param>
        /// <returns>実行結果のメッセージ。</returns>
        private string DoCalc(string inputFilePath,
            string outputFilePath, CalcSettings.CalcRouteMethod method)
        {
            StringBuilder resultMessage = new StringBuilder();

            try
            {
                //---------------------------------------------------------------------------------
                //  管理サービスオブジェクトと環境設定オブジェクトの作成
                //---------------------------------------------------------------------------------
                // 管理サービスオブジェクトを作成
                DCAdmin.ACTDCADMIN adminService = new DCAdmin.ACTDCADMIN();
                // エンドポイントURLを設定
                adminService.Url = user_.AdminUrl;

                // 環境設定オブジェクトを作成
                DCAdmin.Environ adminEnviron = user_.GetAdminEnviron();

                //---------------------------------------------------------------------------------
                //  ユーザー情報オブジェクトの取得
                //---------------------------------------------------------------------------------
                // ユーザー情報オブジェクトを作成
                DCAdmin.UserInformation2 userInfo = new DCAdmin.UserInformation2();
                // ユーザー情報取得サービスの呼び出し
                if (!adminService.GetUserInformation2(adminEnviron, ref userInfo))
                {
                    // ユーザー情報の取得に失敗した場合
                    resultMessage.AppendLine("ユーザー情報の取得に失敗しました。");
                    return resultMessage.ToString();
                }
                user_.UserInfo = userInfo;

                //---------------------------------------------------------------------------------
                //  計算用道路データの取得と選択
                //---------------------------------------------------------------------------------
                // 計算用道路データ取得サービスの呼び出し           
                DCAdmin.NWInfo[] nwInfos = GetNwInfos(user_, adminService, adminEnviron);
                if (nwInfos == null)
                {
                    // 計算用道路データの取得に失敗した場合
                    resultMessage.AppendLine("計算用道路データの取得に失敗しました。");
                    return resultMessage.ToString();
                }

                // 距離計算に使用する計算用道路データを選択
                DCAdmin.NWInfo nwInfo = SelectNwInfo(nwInfos, CalcSettings.NWIDPattern);
                if (nwInfo == null)
                {
                    // 指定の計算用道路データが含まれていない場合
                    resultMessage.AppendLine(
                        "利用可能な計算用道路データに指定のデータが含まれていません。");
                    return resultMessage.ToString();
                }
                resultMessage.AppendLine("計算用道路データ: " + nwInfo.Note);

                //---------------------------------------------------------------------------------
                //  距離計算サービスオブジェクトと環境設定オブジェクトの作成
                //---------------------------------------------------------------------------------
                DCService.ACTDCWS dcService = new DCService.ACTDCWS();
                // 距離計算サーバー取得サービスの呼び出し
                if (!adminService.GetUsableDCServer(ref adminEnviron, nwInfo))
                {
                    // 距離計算サーバーの取得に失敗した場合
                    resultMessage.AppendLine("距離計算サーバーの情報取得に失敗しました。");
                    return resultMessage.ToString();
                }
                // エンドポイントURLを設定
                dcService.Url = Utilities.ChangeProtocol(adminEnviron.WSURL, CalcSettings.UseSsl);
                resultMessage.AppendLine("距離計算サーバー: " + dcService.Url +
                    "(" + adminEnviron.Host + ":" + adminEnviron.Port + ")");

                // 環境設定構造体オブジェクトを作成
                DCService.Environ dcEnvrion = Utilities.GetDCServiceEnviron(adminEnviron);

                //---------------------------------------------------------------------------------
                //  CSVファイルを読み込みロケーション構造体配列を作成
                //---------------------------------------------------------------------------------
                // ファイルから地点の経度緯度情報を読み込む
                CsvRecord[] records = ReadCsvFile(inputFilePath);
                if (records == null)
                {
                    // CSVファイルの読み込みに失敗した場合
                    resultMessage.AppendLine("入力ファイルの読み込みに失敗しました。");
                    return resultMessage.ToString();
                }
                else if (records.Length > user_.UserInfo.CalcRouteMaxLocs)
                {
                    // 地点数が計算可能最大地点数を超えている場合
                    resultMessage.AppendLine("計算可能な最大地点数は"
                        + user_.UserInfo.CalcRouteMaxLocs + "ヶ所です。");
                    return resultMessage.ToString();
                }

                // ロケーション構造体に読み取った経度緯度情報を設定
                DCService.Loc[] locs = CreateLocations(records);

                //---------------------------------------------------------------------------------
                //  ルート計算 / 最短ルート計算の実行
                //---------------------------------------------------------------------------------
                // パラメータ構造体オブジェクトを作成
                DCService.CRParam param = new DCService.CRParam()
                {
                    Datum = (int)CalcSettings.Datum,
                    CoordinateFormat = (int)CalcSettings.CoordinateFormat,
                    CalcKind = (int)CalcSettings.CalcKind,
                    Transport = (int)CalcSettings.Transport,
                    PntsOutput = (int)CalcSettings.PntsOutput,
                    RnsOutput = (int)CalcSettings.RnsOutput,
                    TrdsOutput = (int)CalcSettings.TrdsOutput,
                    HighwayOutput = (int)CalcSettings.HighwayOutput,
                    TollOutput = (int)CalcSettings.TollOutput,
                    RouteKind = (int)CalcSettings.RouteKind,
                };

                // 計算結果を格納する変数を作成
                DCService.Pnt[] pnts = null;
                DCService.Rn[] rns = null;
                DCService.Trd[] trds = null;
                DCService.NWInfo dcnwInfo = null;

                // ルート計算 / 最短ルート計算を実行
                int result = 0;
                switch(method)
                {
                    // ルート計算
                    case CalcSettings.CalcRouteMethod.CalcRoute:
                        result = dcService.CalcRoute(dcEnvrion, param,
                            ref locs, ref pnts, ref rns, ref trds, ref dcnwInfo);
                        break;
                    // 最短ルート計算
                    case CalcSettings.CalcRouteMethod.CalcOptRoute:
                        result = dcService.CalcOptRoute(dcEnvrion, param,
                            ref locs, ref pnts, ref rns, ref trds, ref dcnwInfo);
                        break;
                }
                // 最終距離計算日時を更新
                user_.UpdateLastDCTime();

                //---------------------------------------------------------------------------------
                //  実行結果の取得
                //---------------------------------------------------------------------------------
                // 分単位の所要時間
                int time = locs[locs.Length - 1].Time;
                // m単位の距離をkm単位に換算
                double dist = (double)locs[locs.Length - 1].Dist / (double)1000;

                if (result == 0)
                {
                    // 成功
                    resultMessage.AppendLine(
                        string.Format("成功 所要時間 = {0}分 距離 = {1:0.000}km", time, dist));
                }
                else
                {
                    // 失敗、エラーメッセージを取得
                    resultMessage.AppendLine(string.Format("失敗 {0}{1}", Environment.NewLine,
                        dcService.GetDCWSMessage(dcEnvrion, result)));
                }

                //---------------------------------------------------------------------------------
                //  ファイル出力
                //---------------------------------------------------------------------------------
                if (!WriteLocs(outputFilePath, locs))
                {
                    // 出力に失敗した場合
                    resultMessage.AppendLine("ファイルの書き込みに失敗しました。");
                }
            }
            catch (System.Net.WebException e)
            {
                resultMessage.AppendLine("通信エラーが発生しました。");
                resultMessage.AppendLine(e.Message);
            }
            catch
            {
                //
            }

            return resultMessage.ToString();
        }

        /// <summary>
        /// 計算を実行し実行結果をファイルに出力します。
        /// </summary>
        /// <param name="inputFilePath">入力ファイルのパス</param>
        /// <param name="outputFilePath">出力ファイルのパス</param>
        /// <param name="method">ルート計算の関数</param>
        /// <returns>実行結果のメッセージ。</returns>
        private string DoWork(string inputFilePath, 
            string outputFilePath, CalcSettings.CalcRouteMethod method)
        {
            string resultMessage = string.Empty;
            // 進捗フォームを表示して計算を実行
            ProgressDialog.Run(() =>
                resultMessage = DoCalc(inputFilePath, outputFilePath, method), "計算中");

            return resultMessage;
        }

        /// <summary>
        /// 計算用道路データ取得サービスを呼び出し、利用できる計算用道路データを取得します。
        /// </summary>
        /// <param name="user">計算用道路データ取得サービスに接続するユーザー</param>
        /// <returns></returns>
        private static DCAdmin.NWInfo[] GetNwInfos(User user,
            DCAdmin.ACTDCADMIN adminService, DCAdmin.Environ adminEnviron)
        {
            // 取得した計算用道路データを格納する配列を用意
            DCAdmin.NWInfo[] nwInfos = null;
            // 計算用道路データ取得サービスを呼び出す
            adminService.GetUsableNWInfos(adminEnviron, ref nwInfos);

            return nwInfos;
        }

        /// <summary>
        /// 指定した正規表現パターンとNWIDが一致するデータを返します。
        /// </summary>
        /// <param name="nwInfos">対象のNWInfo配列</param>
        /// <param name="pattern">NWIDの検索に使用する正規表現パターン</param>
        /// <returns>NWIDが一致したNWInfo。一致するNWInfoがない場合はnull。</returns>
        private static DCAdmin.NWInfo SelectNwInfo(DCAdmin.NWInfo[] nwInfos, string pattern)
        {
            // NWIDを正規表現で検索するクエリ
            var nwInfoQuery =
                from nwInfo in nwInfos
                where Regex.IsMatch(nwInfo.NWID.ToString(), CalcSettings.NWIDPattern)
                orderby nwInfo.NWID descending
                select nwInfo;

            // クエリの一番先頭を返す
            return nwInfoQuery.FirstOrDefault();
        }

        /// <summary>
        /// CSVファイルの全レコードを読み取り結果を返します。
        /// </summary>
        /// <param name="path">対象ファイルのパス</param>
        /// <returns>読み取り結果のCsvRecord配列。読み取りに失敗した場合はnull。</returns>
        private static CsvRecord[] ReadCsvFile(string path)
        {
            CsvRecord[] records = null;
            try
            {
                using (StreamReader strmReader =
                    new StreamReader(path, Encoding.GetEncoding("shift_jis")))
                using (CsvReader csvReader =
                    new CsvReader(strmReader, false, Delimiter.Comma, QuotationMark.Double))
                {
                    csvReader.RemoveEscapeDoubleQuotes = false;
                    records = csvReader.ReadAllRecords();
                }
            }
            catch
            {
                //
            }

            return records;
        }

        /// <summary>
        /// CsvRecord配列からロケーション構造体配列を作成し返します。
        /// </summary>
        /// <param name="records">CsvRecord配列</param>
        /// <returns>ロケーション構造体配列。</returns>
        private static DCService.Loc[] CreateLocations(CsvRecord[] records)
        {
            // ロケーション構造体に経度緯度情報を設定
            DCService.Loc[] locations = new DCService.Loc[records.Length];
            for (int i = 0; i < records.Length; i++)
            {
                locations[i] = new DCService.Loc();
                locations[i].Lon = records[i][LonColumnIndex];
                locations[i].Lat = records[i][LatColumnIndex];
                locations[i].Tag = "\"" + records[i][NameColumnIndex] + "\"";
                locations[i].Tag += ",\"" + records[i][AddressColumnIndex] + "\"";
            }

            return locations;
        }

        /// <summary>
        /// ロケーション構造体の所要時間と距離をCSVファイルに書き込みます。
        /// </summary>
        /// <param name="path">出力ファイルのパス</param>
        /// <param name="locs">計算結果を格納したロケーション構造体配列</param>
        /// <returns>書き込みに成功した場合はtrue、失敗した場合はfalse。</returns>
        private static bool WriteLocs(string path, DCService.Loc[] locs)
        {
            try
            {
                using (StreamWriter writer = 
                    new StreamWriter(path, false, Encoding.GetEncoding("shift_jis")))
                {
                    for (int i = 0; i < locs.Length; i++)
                    {
                        writer.Write(locs[i].Tag);
                        writer.Write(",\"" + locs[i].Lon + "\"");
                        writer.Write(",\"" + locs[i].Lat + "\"");
                        writer.Write(",\"" + locs[i].Time + "\"");
                        writer.WriteLine(",\"" + ((double)locs[i].Dist / (double)1000) + "\"");
                    }
                }
            }
            catch
            {
                return false;
            }

            return true;
        }

        /// <summary>
        /// コントロールからユーザー設定を取得します。
        /// </summary>
        /// <returns>取得したUserSettingsオブジェクト。</returns>
        private UserSettings GetUserSettingsFromControls()
        {
            UserSettings settings = new UserSettings();
            settings.UserID = textBoxUserId.Text;
            settings.Password = textBoxPassword.Text;
            settings.AdminUrl = textBoxAdminServerUrl.Text;
            settings.InputFilePath = textBoxInputFilePath.Text;
            settings.OutputFilePath = textBoxOutputFilePath.Text;
            if (radioButtonCalcOptRoute.Checked)
            {
                settings.CalcRouteMethod = CalcSettings.CalcRouteMethod.CalcOptRoute;
            }

            return settings;
        }

        /// <summary>
        /// 指定したユーザー設定をコントロールに適用します。
        /// </summary>
        /// <param name="settings">適用するUserSettingsオブジェクト</param>
        private void ApplyUserSettingsToControls(UserSettings settings)
        {
            textBoxUserId.Text = settings.UserID;
            textBoxAdminServerUrl.Text = settings.AdminUrl;
            textBoxInputFilePath.Text = settings.InputFilePath;
            textBoxOutputFilePath.Text = settings.OutputFilePath;
            if (settings.CalcRouteMethod == CalcSettings.CalcRouteMethod.CalcOptRoute)
            {
                radioButtonCalcOptRoute.Checked = true;
            }
        }

        /// <summary>
        /// 入力ファイルを選択し、選択したファイルのパスを返します。
        /// </summary>
        /// <param name="oldPath">現在の入力ファイルパス</param>
        /// <returns>選択したファイルのパス。キャンセルの場合はnull。</returns>
        private static string SelectInputFile(string oldPath)
        {
            string newPath = null;
            using (OpenFileDialog dialog = new OpenFileDialog())
            {
                if (File.Exists(oldPath))
                {
                    dialog.InitialDirectory = Path.GetDirectoryName(oldPath);
                    dialog.FileName = Path.GetFileName(oldPath);
                }
                dialog.Title = "入力ファイルの選択";
                dialog.Filter = "CSVファイル (*.csv)|*.csv|全てのファイル (*.*)|*.*";

                if (dialog.ShowDialog() == DialogResult.OK)
                {
                    newPath = dialog.FileName;
                }
            }

            return newPath;
        }

        /// <summary>
        /// 出力ファイルを選択し、選択したファイルのパスを返します。
        /// </summary>
        /// <param name="oldPath">現在の出力ファイルパス</param>
        /// <returns>選択したファイルのパス。キャンセルの場合はnull。</returns>
        private static string SelectOutputFile(string oldPath)
        {
            string newPath = null;
            using (SaveFileDialog dialog = new SaveFileDialog())
            {
                if (File.Exists(oldPath))
                {
                    dialog.InitialDirectory = Path.GetDirectoryName(oldPath);
                    dialog.FileName = Path.GetFileName(oldPath);
                }
                dialog.Title = "出力ファイルの選択";
                dialog.Filter = "CSVファイル (*.csv)|*.csv|全てのファイル (*.*)|*.*";

                if (dialog.ShowDialog() == DialogResult.OK)
                {
                    newPath = dialog.FileName;
                }
            }

            return newPath;
        }

        /// <summary>
        /// ユーザー設定を検証します。
        /// </summary>
        /// <param name="userID">ユーザーID</param>
        /// <param name="password">パスワード</param>
        /// <param name="adminUrl">管理サーバーURL</param>
        /// <param name="inputFilePath">入力ファイルのパス</param>
        /// <param name="outputFilePath">出力ファイルのパス</param>
        /// <returns>設定が全て有効な場合はnull、それ以外の場合はエラーメッセージ。</returns>
        private static string ValidateSettings(string userID, string password, string adminUrl,
            string inputFilePath, string outputFilePath)
        {
            StringBuilder message = new StringBuilder();
            if (string.IsNullOrEmpty(userID))
            {
                message.AppendLine("ユーザーIDが指定されていません。");
            }
            if (string.IsNullOrEmpty(password))
            {
                message.AppendLine("パスワードが指定されていません。");
            }
            if (string.IsNullOrEmpty(adminUrl))
            {
                message.AppendLine("管理サーバーURLが指定されていません。");
            }
            string inputFilePathError = ValidateInputFilePath(inputFilePath);
            if (inputFilePathError != null)
            {
                message.AppendLine(inputFilePathError);
            }
            string outputFilePathError = ValidateOutputFilePath(outputFilePath);
            if (outputFilePathError != null)
            {
                message.AppendLine(outputFilePathError);
            }
            if (message.Length == 0)
            {
                if (ConfirmOverwrite(outputFilePath))
                {
                    return null;
                }
                message.AppendLine("指定された " + outputFilePath + " は既に存在します。");
            }

            return message.ToString();
        }

        /// <summary>
        /// 入力ファイルのパスを検証します。
        /// </summary>
        /// <param name="path">検査対象のパス</param>
        /// <returns>有効な場合はnull、それ以外の場合はエラーメッセージ。</returns>
        private static string ValidateInputFilePath(string path)
        {
            if (string.IsNullOrEmpty(path))
            {
                return "入力ファイルが指定されていません。";
            }
            try
            {
                if (!File.Exists(path))
                {
                    return "指定された " + path + " が存在しません。";
                }
            }
            catch
            {
                return "入力ファイルのパスが不正です。";
            }

            return null;
        }

        /// <summary>
        /// 出力ファイルのパスを検証します。
        /// </summary>
        /// <param name="path">検査対象のパス</param>
        /// <returns>有効な場合はnull、それ以外の場合はエラーメッセージ。</returns>
        private static string ValidateOutputFilePath(string path)
        {
            if (string.IsNullOrEmpty(path))
            {
                return "出力ファイルが指定されていません。";
            }
            try
            {
                string directoryName = Path.GetDirectoryName(path);
                if (string.IsNullOrEmpty(directoryName))
                {
                    return "指定されたフォルダが存在しません。";
                }
                if (!Directory.Exists(directoryName))
                {
                    return "指定された " + directoryName + " が存在しません。";
                }
            }
            catch
            {
                return "出力ファイルのパスが不正です。";
            }

            return null;
        }

        /// <summary>
        /// ファイルが既に存在する場合に上書き確認を行います。
        /// </summary>
        /// <param name="path">検査対象のパス</param>
        /// <returns>ファイルが既に存在していて上書きしない場合はtrue、それ以外の場合はfalse。</returns>
        private static bool ConfirmOverwrite(string path)
        {
            if (File.Exists(path))
            {
                DialogResult result = MessageBox.Show(path + " は既に存在しています。" +
                    Environment.NewLine + "上書きしてよろしいですか？", "上書き確認",
                    MessageBoxButtons.YesNo, MessageBoxIcon.Exclamation,
                    MessageBoxDefaultButton.Button2);
                if (result == DialogResult.No)
                {
                    return false;
                }
            }

            return true;
        }

        /// <summary>
        /// 再計算可能までの残待機時間を更新します。
        /// </summary>
        /// <param name="waitingSeconds">秒単位の残り待機時間</param>
        private void UpdateWaitingTime(int waitingSeconds)
        {
            string labelText = string.Empty;
            if (waitingSeconds > 0)
            {
                // 待機時間が経過していない場合
                labelText = string.Format("再計算可能まで {0:00} 秒", waitingSeconds);
            }
            // コントロールの更新
            labelCountDown.Text = labelText;
            buttonCalc.Enabled = waitingSeconds <= 0;
            timerCalcWait.Enabled = waitingSeconds > 0;
        }

        #endregion

        //=========================================================================================
        //  Event Handlers
        //=========================================================================================
        #region Event Handlers

        /// <summary>
        /// メインフォームのロードイベントです。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainForm_Load(object sender, EventArgs e)
        {
            // 保存したユーザー設定を復元し設定する
            userSettings_ = UserSettings.Load(UserSettings.SettingFilePath);
            if (userSettings_ == null)
            {
                // 失敗した場合は既定値を設定
                userSettings_ = new UserSettings();
                textBoxAdminServerUrl.Text = CalcSettings.DefaultAdminUrl;
                return;
            }
            ApplyUserSettingsToControls(userSettings_);
        }

        /// <summary>
        /// メインフォームのクローズ後イベントです。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainForm_FormClosed(object sender, FormClosedEventArgs e)
        {
            // ユーザー設定を保存する
            userSettings_ = GetUserSettingsFromControls();
            try
            {
                userSettings_.Save(UserSettings.SettingFilePath);
            }
            catch
            {
                //
            }
        }

        /// <summary>
        /// 「既定値」ボタンのクリックイベントです。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonResetUrl_Click(object sender, EventArgs e)
        {
            textBoxAdminServerUrl.Text = CalcSettings.DefaultAdminUrl;
        }

        /// <summary>
        /// 「参照」ボタンのクリックイベントです。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonSelectInputFile_Click(object sender, EventArgs e)
        {
            string path = SelectInputFile(textBoxInputFilePath.Text) ?? textBoxInputFilePath.Text;
            textBoxInputFilePath.Text = path;
        }

        /// <summary>
        /// 「参照」ボタンのクリックイベントです。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonSelectOutputFile_Click(object sender, EventArgs e)
        {
            string path = SelectOutputFile(textBoxOutputFilePath.Text) ?? textBoxOutputFilePath.Text;
            textBoxOutputFilePath.Text = path;
        }

        /// <summary>
        /// 「計算」ボタンのクリックイベントです。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonCalc_Click(object sender, EventArgs e)
        {
            // フォームから設定を取得し検証
            userSettings_ = GetUserSettingsFromControls();
            string errorMessage = ValidateSettings(userSettings_.UserID, userSettings_.Password,
                userSettings_.AdminUrl, userSettings_.InputFilePath, userSettings_.OutputFilePath); 
            if (errorMessage != null)
            {
                textBoxResults.Text = errorMessage;
                return;
            }

            // テキストボックスの表示を変更
            textBoxResults.Text = "計算中...";

            // 「計算」ボタンを無効
            buttonCalc.Enabled = false;

            // AdminURLを確認してSSL使用フラグを変更
            CalcSettings.UseSsl = userSettings_.AdminUrl.StartsWith("https") == true;

            // ユーザーを作成
            user_ = new User(userSettings_.UserID, userSettings_.Password,
                userSettings_.AdminUrl, CalcSettings.UseSsl);

            // 計算の実行
            textBoxResults.Text = DoWork(userSettings_.InputFilePath,
                userSettings_.OutputFilePath, userSettings_.CalcRouteMethod);

            // 再計算までのカウントダウン
            timerCalcWait.Start();
        }

        /// <summary>
        /// 「終了」ボタンのクリックイベントです。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonExit_Click(object sender, EventArgs e)
        {
            Close();
        }

        /// <summary>
        /// 「マニュアルを開く」リンクのクリックイベントです。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void linkLabelManual_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            try
            {
                Process.Start(UserSettings.UsersGuideFilePath);
            }
            catch
            {

            }
        }

        /// <summary>
        /// 計算待機時間タイマーのイベントです。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void timerCalcWait_Tick(object sender, EventArgs e)
        {
            if (user_.UserInfo == null)
            {
                UpdateWaitingTime(-1);
                return;
            }
            int elpsedSeconds = (int)((DateTime.Now - user_.LastDCTime).TotalSeconds);
            int waitingSeconds = user_.UserInfo.CalcInterval - elpsedSeconds;
            UpdateWaitingTime(waitingSeconds);
        }

        #endregion
    }
}
